--enemy prototype sample
--love 0.8.0
function love.load()
  love.graphics.setDefaultImageFilter("nearest", "nearest")
  DRAWING_SCALE = 2
  
  BOX_X, BOX_Y, BOX_W, BOX_H = 30, 30, 500, 400
  function drawBox()
    love.graphics.rectangle("line", BOX_X, BOX_Y, BOX_W, BOX_H)
  end
  
  tImage = {}
  for i, name in ipairs({
    "peony";
    "tarmac";
    "navy";
  }) do
    tImage[name] = love.graphics.newImage(string.format("image/%s.png", name))
  end
  
  enemyPrototype = {}
  function enemyPrototype:new(x, y, vx, vy, ax, ay, image)
    local enemyInstance = {
      x = x or self.x;
      y = y or self.y;
      vx = vx or self.vx;
      vy = vy or self.vy;
      ax = ax or self.ax;
      ay = ay or self.ay;
      image = image or self.image;
      imageSize = image and image:getWidth() or self.image:getWidth();
    }
    return setmetatable(enemyInstance, {__index = self})
  end
  function enemyPrototype:update(dt)
    self.x = self.x + self.vx*dt
    self.y = self.y + self.vy*dt
    self.vx = self.vx + self.ax*dt
    self.vy = self.vy + self.ay*dt
    self.angle = math.atan(self.vy/self.vx) - math.rad(90)
  end
  function enemyPrototype:draw(dx, dy)
    love.graphics.draw(self.image, self.x + dx, self.y + dy, self.angle, DRAWING_SCALE)
  end
  
  enemyManager = {
    tEnemy = {}
  }
  function enemyManager:spawn(prototype, amount, ...)
    for i = 1, (amount or 1) do
      table.insert(self.tEnemy, prototype:new(...))
    end
  end
  function enemyManager:update(dt)
    for k, enemy in pairs(self.tEnemy) do
      enemy:update(dt)
      --remove enemy going out of the box
      local x = enemy.x + enemy.imageSize
      local y = enemy.y + enemy.imageSize
      if x < 0 or x > BOX_W or y < 0 or y > BOX_H then
        self.tEnemy[k] = nil
      end
    end
  end
  function enemyManager:draw()
    for k, enemy in pairs(self.tEnemy) do
      enemy:draw(BOX_X, BOX_Y)
    end
  end
  --debug
  function enemyManager:print(x, y)
    drawBox()
    for k, enemy in pairs(self.tEnemy) do
      love.graphics.draw(enemy.image, x + 18, y - 1)
      love.graphics.print(string.format("[%d]    (%d, %d) vx = %d; vy = %d", k, enemy.x, enemy.y, enemy.vx, enemy.vy), x, y); y = y + 15
      love.graphics.print(k, enemy.x + BOX_X, enemy.y + BOX_Y)
    end
  end
  
  redEnemyPrototype = enemyPrototype:new(450, 50, 0, 0, -20, 0, tImage.peony)
  greenUpEnemyPrototype = enemyPrototype:new(450, 150, -30, -20, -10, -10, tImage.tarmac)
  greenDownEnemyPrototype = enemyPrototype:new(450, 150, -30, 20, -10, 30, tImage.tarmac)
  blueEnemyPrototype = enemyPrototype:new(450, 250, -30, 0, -5, 0, tImage.navy)
  
  enemyManager:spawn(redEnemyPrototype)
  enemyManager:spawn(redEnemyPrototype, 1, 300, 50)
  enemyManager:spawn(greenUpEnemyPrototype)
  enemyManager:spawn(greenDownEnemyPrototype)
  enemyManager:spawn(blueEnemyPrototype)
end

function love.update(dt)
  enemyManager:update(dt)
end

function love.draw()
  enemyManager:draw()
  enemyManager:print(BOX_X, BOX_Y)
end

function love.keypressed(key)
  if key == "f5" then
    love.filesystem.load("main.lua")()
    love.load()
  elseif key == "escape" then
    love.event.push("quit")
  end
end
